@@ -18,6 +18,7 @@
 use crate::ffi;
 use foreign_types::{ForeignType, ForeignTypeRef};
 use libc::c_int;
+use openssl_macros::corresponds;
 use std::fmt;
 use std::ptr;
 
@@ -111,10 +112,7 @@ foreign_type_and_impl_send_sync! {
 
 impl EcGroup {
     /// Returns the group of a standard named curve.
-    ///
-    /// OpenSSL documentation at [`EC_GROUP_new`].
-    ///
-    /// [`EC_GROUP_new`]: https://www.openssl.org/docs/man1.1.0/crypto/EC_GROUP_new.html
+    #[corresponds(EC_GROUP_new)]
     pub fn from_curve_name(nid: Nid) -> Result<EcGroup, ErrorStack> {
         unsafe {
             init();
@@ -150,10 +148,7 @@ impl EcGroupRef {
     }
 
     /// Places the cofactor of the group in the provided `BigNum`.
-    ///
-    /// OpenSSL documentation at [`EC_GROUP_get_cofactor`]
-    ///
-    /// [`EC_GROUP_get_cofactor`]: https://www.openssl.org/docs/man1.1.0/crypto/EC_GROUP_get_cofactor.html
+    #[corresponds(EC_GROUP_get_cofactor)]
     pub fn cofactor(
         &self,
         cofactor: &mut BigNumRef,
@@ -170,29 +165,20 @@ impl EcGroupRef {
     }
 
     /// Returns the degree of the curve.
-    ///
-    /// OpenSSL documentation at [`EC_GROUP_get_degree`]
-    ///
-    /// [`EC_GROUP_get_degree`]: https://www.openssl.org/docs/man1.1.0/crypto/EC_GROUP_get_degree.html
+    #[corresponds(EC_GROUP_get_degree)]
     #[allow(clippy::unnecessary_cast)]
     pub fn degree(&self) -> u32 {
         unsafe { ffi::EC_GROUP_get_degree(self.as_ptr()) as u32 }
     }
 
     /// Returns the number of bits in the group order.
-    ///
-    /// OpenSSL documentation at [`EC_GROUP_order_bits`]
-    ///
-    /// [`EC_GROUP_order_bits`]: https://www.openssl.org/docs/man1.1.0/crypto/EC_GROUP_order_bits.html
+    #[corresponds(EC_GROUP_order_bits)]
     pub fn order_bits(&self) -> u32 {
         unsafe { ffi::EC_GROUP_order_bits(self.as_ptr()) as u32 }
     }
 
     /// Returns the generator for the given curve as a [`EcPoint`].
-    ///
-    /// OpenSSL documentation at [`EC_GROUP_get0_generator`]
-    ///
-    /// [`EC_GROUP_get0_generator`]: https://www.openssl.org/docs/man1.1.0/man3/EC_GROUP_get0_generator.html
+    #[corresponds(EC_GROUP_get0_generator)]
     pub fn generator(&self) -> &EcPointRef {
         unsafe {
             let ptr = ffi::EC_GROUP_get0_generator(self.as_ptr());
@@ -201,10 +187,7 @@ impl EcGroupRef {
     }
 
     /// Places the order of the curve in the provided `BigNum`.
-    ///
-    /// OpenSSL documentation at [`EC_GROUP_get_order`]
-    ///
-    /// [`EC_GROUP_get_order`]: https://www.openssl.org/docs/man1.1.0/crypto/EC_GROUP_get_order.html
+    #[corresponds(EC_GROUP_get_order)]
     pub fn order(
         &self,
         order: &mut BigNumRef,
@@ -232,10 +215,7 @@ impl EcGroupRef {
     }
 
     /// Returns the name of the curve, if a name is associated.
-    ///
-    /// OpenSSL documentation at [`EC_GROUP_get_curve_name`]
-    ///
-    /// [`EC_GROUP_get_curve_name`]: https://www.openssl.org/docs/man1.1.0/crypto/EC_GROUP_get_curve_name.html
+    #[corresponds(EC_GROUP_get_curve_name)]
     pub fn curve_name(&self) -> Option<Nid> {
         let nid = unsafe { ffi::EC_GROUP_get_curve_name(self.as_ptr()) };
         if nid > 0 {
@@ -260,10 +240,7 @@ foreign_type_and_impl_send_sync! {
 
 impl EcPointRef {
     /// Computes `a + b`, storing the result in `self`.
-    ///
-    /// OpenSSL documentation at [`EC_POINT_add`]
-    ///
-    /// [`EC_POINT_add`]: https://www.openssl.org/docs/man1.1.0/crypto/EC_POINT_add.html
+    #[corresponds(EC_POINT_add)]
     pub fn add(
         &mut self,
         group: &EcGroupRef,
@@ -284,10 +261,7 @@ impl EcPointRef {
     }
 
     /// Computes `q * m`, storing the result in `self`.
-    ///
-    /// OpenSSL documentation at [`EC_POINT_mul`]
-    ///
-    /// [`EC_POINT_mul`]: https://www.openssl.org/docs/man1.1.0/crypto/EC_POINT_mul.html
+    #[corresponds(EC_POINT_mul)]
     pub fn mul(
         &mut self,
         group: &EcGroupRef,
@@ -353,10 +327,7 @@ impl EcPointRef {
     }
 
     /// Inverts `self`.
-    ///
-    /// OpenSSL documentation at [`EC_POINT_invert`]
-    ///
-    /// [`EC_POINT_invert`]: https://www.openssl.org/docs/man1.1.0/crypto/EC_POINT_invert.html
+    #[corresponds(EC_POINT_invert)]
     pub fn invert(&mut self, group: &EcGroupRef, ctx: &BigNumContextRef) -> Result<(), ErrorStack> {
         unsafe {
             cvt(ffi::EC_POINT_invert(
@@ -369,10 +340,7 @@ impl EcPointRef {
     }
 
     /// Serializes the point to a binary representation.
-    ///
-    /// OpenSSL documentation at [`EC_POINT_point2oct`]
-    ///
-    /// [`EC_POINT_point2oct`]: https://www.openssl.org/docs/man1.1.0/crypto/EC_POINT_point2oct.html
+    #[corresponds(EC_POINT_point2oct)]
     pub fn to_bytes(
         &self,
         group: &EcGroupRef,
@@ -409,10 +377,7 @@ impl EcPointRef {
     }
 
     /// Creates a new point on the specified curve with the same value.
-    ///
-    /// OpenSSL documentation at [`EC_POINT_dup`]
-    ///
-    /// [`EC_POINT_dup`]: https://www.openssl.org/docs/man1.1.0/crypto/EC_POINT_dup.html
+    #[corresponds(EC_POINT_dup)]
     pub fn to_owned(&self, group: &EcGroupRef) -> Result<EcPoint, ErrorStack> {
         unsafe {
             cvt_p(ffi::EC_POINT_dup(self.as_ptr(), group.as_ptr())).map(|p| EcPoint::from_ptr(p))
@@ -443,10 +408,7 @@ impl EcPointRef {
 
     /// Place affine coordinates of a curve over a prime field in the provided
     /// `x` and `y` `BigNum`s
-    ///
-    /// OpenSSL documentation at [`EC_POINT_get_affine_coordinates_GFp`]
-    ///
-    /// [`EC_POINT_get_affine_coordinates_GFp`]: https://www.openssl.org/docs/man1.1.0/crypto/EC_POINT_get_affine_coordinates_GFp.html
+    #[corresponds(EC_POINT_get_affine_coordinates_GFp)]
     pub fn affine_coordinates_gfp(
         &self,
         group: &EcGroupRef,
@@ -469,19 +431,13 @@ impl EcPointRef {
 
 impl EcPoint {
     /// Creates a new point on the specified curve.
-    ///
-    /// OpenSSL documentation at [`EC_POINT_new`]
-    ///
-    /// [`EC_POINT_new`]: https://www.openssl.org/docs/man1.1.0/crypto/EC_POINT_new.html
+    #[corresponds(EC_POINT_new)]
     pub fn new(group: &EcGroupRef) -> Result<EcPoint, ErrorStack> {
         unsafe { cvt_p(ffi::EC_POINT_new(group.as_ptr())).map(|p| EcPoint::from_ptr(p)) }
     }
 
     /// Creates point from a binary representation
-    ///
-    /// OpenSSL documentation at [`EC_POINT_oct2point`]
-    ///
-    /// [`EC_POINT_oct2point`]: https://www.openssl.org/docs/man1.1.0/crypto/EC_POINT_oct2point.html
+    #[corresponds(EC_POINT_oct2point)]
     pub fn from_bytes(
         group: &EcGroupRef,
         buf: &[u8],
@@ -507,9 +463,6 @@ generic_foreign_type_and_impl_send_sync! {
 
     /// Public and optional Private key on the given curve
     ///
-    /// OpenSSL documentation at [`EC_KEY_new`]
-    ///
-    /// [`EC_KEY_new`]: https://www.openssl.org/docs/man1.1.0/crypto/EC_KEY_new.html
     pub struct EcKey<T>;
 
     /// Reference to [`EcKey`]
@@ -526,37 +479,25 @@ where
         /// Serializes the private key to a PEM-encoded ECPrivateKey structure.
         ///
         /// The output will have a header of `-----BEGIN EC PRIVATE KEY-----`.
-        ///
-        /// This corresponds to [`PEM_write_bio_ECPrivateKey`].
-        ///
-        /// [`PEM_write_bio_ECPrivateKey`]: https://www.openssl.org/docs/man1.1.0/crypto/PEM_write_bio_ECPrivateKey.html
+        #[corresponds(PEM_write_bio_ECPrivateKey)]
         private_key_to_pem,
         /// Serializes the private key to a PEM-encoded encrypted ECPrivateKey structure.
         ///
         /// The output will have a header of `-----BEGIN EC PRIVATE KEY-----`.
-        ///
-        /// This corresponds to [`PEM_write_bio_ECPrivateKey`].
-        ///
-        /// [`PEM_write_bio_ECPrivateKey`]: https://www.openssl.org/docs/man1.1.0/crypto/PEM_write_bio_ECPrivateKey.html
+        #[corresponds(PEM_write_bio_ECPrivateKey)]
         private_key_to_pem_passphrase,
         ffi::PEM_write_bio_ECPrivateKey
     }
 
     to_der! {
         /// Serializes the private key into a DER-encoded ECPrivateKey structure.
-        ///
-        /// This corresponds to [`i2d_ECPrivateKey`].
-        ///
-        /// [`i2d_ECPrivateKey`]: https://www.openssl.org/docs/man1.0.2/crypto/d2i_ECPrivate_key.html
+        #[corresponds(i2d_ECPrivateKey)]
         private_key_to_der,
         ffi::i2d_ECPrivateKey
     }
 
     /// Return [`EcPoint`] associated with the private key
-    ///
-    /// OpenSSL documentation at [`EC_KEY_get0_private_key`]
-    ///
-    /// [`EC_KEY_get0_private_key`]: https://www.openssl.org/docs/man1.1.0/crypto/EC_KEY_get0_private_key.html
+    #[corresponds(EC_KEY_get0_private_key)]
     pub fn private_key(&self) -> &BigNumRef {
         unsafe {
             let ptr = ffi::EC_KEY_get0_private_key(self.as_ptr());
@@ -570,10 +511,7 @@ where
     T: HasPublic,
 {
     /// Returns the public key.
-    ///
-    /// OpenSSL documentation at [`EC_KEY_get0_public_key`]
-    ///
-    /// [`EC_KEY_get0_public_key`]: https://www.openssl.org/docs/man1.1.0/crypto/EC_KEY_get0_public_key.html
+    #[corresponds(EC_KEY_get0_public_key)]
     pub fn public_key(&self) -> &EcPointRef {
         unsafe {
             let ptr = ffi::EC_KEY_get0_public_key(self.as_ptr());
@@ -585,20 +523,14 @@ where
         /// Serialies the public key into a PEM-encoded SubjectPublicKeyInfo structure.
         ///
         /// The output will have a header of `-----BEGIN PUBLIC KEY-----`.
-        ///
-        /// This corresponds to [`PEM_write_bio_EC_PUBKEY`].
-        ///
-        /// [`PEM_write_bio_EC_PUBKEY`]: https://www.openssl.org/docs/man1.1.0/crypto/PEM_write_bio_EC_PUBKEY.html
+        #[corresponds(PEM_write_bio_EC_PUBKEY)]
         public_key_to_pem,
         ffi::PEM_write_bio_EC_PUBKEY
     }
 
     to_der! {
         /// Serializes the public key into a DER-encoded SubjectPublicKeyInfo structure.
-        ///
-        /// This corresponds to [`i2d_EC_PUBKEY`].
-        ///
-        /// [`i2d_EC_PUBKEY`]: https://www.openssl.org/docs/man1.1.0/crypto/i2d_EC_PUBKEY.html
+        #[corresponds(i2d_EC_PUBKEY)]
         public_key_to_der,
         ffi::i2d_EC_PUBKEY
     }
@@ -609,10 +541,7 @@ where
     T: HasParams,
 {
     /// Return [`EcGroup`] of the `EcKey`
-    ///
-    /// OpenSSL documentation at [`EC_KEY_get0_group`]
-    ///
-    /// [`EC_KEY_get0_group`]: https://www.openssl.org/docs/man1.1.0/crypto/EC_KEY_get0_group.html
+    #[corresponds(EC_KEY_get0_group)]
     pub fn group(&self) -> &EcGroupRef {
         unsafe {
             let ptr = ffi::EC_KEY_get0_group(self.as_ptr());
@@ -621,10 +550,7 @@ where
     }
 
     /// Checks the key for validity.
-    ///
-    /// OpenSSL documentation at [`EC_KEY_check_key`]
-    ///
-    /// [`EC_KEY_check_key`]: https://www.openssl.org/docs/man1.1.0/crypto/EC_KEY_check_key.html
+    #[corresponds(EC_KEY_check_key)]
     pub fn check_key(&self) -> Result<(), ErrorStack> {
         unsafe { cvt(ffi::EC_KEY_check_key(self.as_ptr())).map(|_| ()) }
     }
@@ -647,10 +573,7 @@ impl EcKey<Params> {
     ///
     /// It will not have an associated public or private key. This kind of key is primarily useful
     /// to be provided to the `set_tmp_ecdh` methods on `Ssl` and `SslContextBuilder`.
-    ///
-    /// OpenSSL documentation at [`EC_KEY_new_by_curve_name`]
-    ///
-    /// [`EC_KEY_new_by_curve_name`]: https://www.openssl.org/docs/man1.1.0/crypto/EC_KEY_new_by_curve_name.html
+    #[corresponds(EC_KEY_new_by_curve_name)]
     pub fn from_curve_name(nid: Nid) -> Result<EcKey<Params>, ErrorStack> {
         unsafe {
             init();
@@ -659,10 +582,7 @@ impl EcKey<Params> {
     }
 
     /// Constructs an `EcKey` corresponding to a curve.
-    ///
-    /// This corresponds to [`EC_KEY_set_group`].
-    ///
-    /// [`EC_KEY_set_group`]: https://www.openssl.org/docs/man1.1.0/crypto/EC_KEY_new.html
+    #[corresponds(EC_KEY_set_group)]
     pub fn from_group(group: &EcGroupRef) -> Result<EcKey<Params>, ErrorStack> {
         unsafe {
             cvt_p(ffi::EC_KEY_new())
@@ -743,10 +663,7 @@ impl EcKey<Public> {
         /// Decodes a PEM-encoded SubjectPublicKeyInfo structure containing a EC key.
         ///
         /// The input should have a header of `-----BEGIN PUBLIC KEY-----`.
-        ///
-        /// This corresponds to [`PEM_read_bio_EC_PUBKEY`].
-        ///
-        /// [`PEM_read_bio_EC_PUBKEY`]: https://www.openssl.org/docs/man1.1.0/crypto/PEM_read_bio_EC_PUBKEY.html
+        #[corresponds(PEM_read_bio_EC_PUBKEY)]
         public_key_from_pem,
         EcKey<Public>,
         ffi::PEM_read_bio_EC_PUBKEY
@@ -754,10 +671,7 @@ impl EcKey<Public> {
 
     from_der! {
         /// Decodes a DER-encoded SubjectPublicKeyInfo structure containing a EC key.
-        ///
-        /// This corresponds to [`d2i_EC_PUBKEY`].
-        ///
-        /// [`d2i_EC_PUBKEY`]: https://www.openssl.org/docs/man1.1.0/crypto/d2i_EC_PUBKEY.html
+        #[corresponds(d2i_EC_PUBKEY)]
         public_key_from_der,
         EcKey<Public>,
         ffi::d2i_EC_PUBKEY,
@@ -811,15 +725,13 @@ impl EcKey<Private> {
         /// Deserializes a private key from a PEM-encoded ECPrivateKey structure.
         ///
         /// The input should have a header of `-----BEGIN EC PRIVATE KEY-----`.
-        ///
-        /// This corresponds to `PEM_read_bio_ECPrivateKey`.
+        #[corresponds(PEM_read_bio_ECPrivateKey)]
         private_key_from_pem,
 
         /// Deserializes a private key from a PEM-encoded encrypted ECPrivateKey structure.
         ///
         /// The input should have a header of `-----BEGIN EC PRIVATE KEY-----`.
-        ///
-        /// This corresponds to `PEM_read_bio_ECPrivateKey`.
+        #[corresponds(PEM_read_bio_ECPrivateKey)]
         private_key_from_pem_passphrase,
 
         /// Deserializes a private key from a PEM-encoded encrypted ECPrivateKey structure.
@@ -827,8 +739,7 @@ impl EcKey<Private> {
         /// The callback should fill the password into the provided buffer and return its length.
         ///
         /// The input should have a header of `-----BEGIN EC PRIVATE KEY-----`.
-        ///
-        /// This corresponds to `PEM_read_bio_ECPrivateKey`.
+        #[corresponds(PEM_read_bio_ECPrivateKey)]
         private_key_from_pem_callback,
         EcKey<Private>,
         ffi::PEM_read_bio_ECPrivateKey
@@ -836,10 +747,7 @@ impl EcKey<Private> {
 
     from_der! {
         /// Decodes a DER-encoded elliptic curve private key structure.
-        ///
-        /// This corresponds to [`d2i_ECPrivateKey`].
-        ///
-        /// [`d2i_ECPrivateKey`]: https://www.openssl.org/docs/man1.0.2/crypto/d2i_ECPrivate_key.html
+        #[corresponds(d2i_ECPrivateKey)]
         private_key_from_der,
         EcKey<Private>,
         ffi::d2i_ECPrivateKey,
